/*
   This file is part of TALER
   Copyright (C) 2015-2026 Taler Systems SA

   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
   Foundation; either version 3, or (at your option) any later version.

   TALER is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   TALER; see the file COPYING.  If not, see
   <http://www.gnu.org/licenses/>
 */
/**
 * @file lib/exchange_api_post-management-extensions.c
 * @brief functions to handle the settings for extensions (p2p and age restriction)
 * @author Özgür Kesim
 */
#include "taler/platform.h"
#include "taler/taler_json_lib.h"
#include <gnunet/gnunet_curl_lib.h>
#include "taler/taler_extensions.h"
#include "taler/taler_exchange_service.h"
#include "taler/taler-exchange/post-management-extensions.h"
#include "exchange_api_curl_defaults.h"
#include "taler/taler_signatures.h"
#include "taler/taler_curl_lib.h"


/**
 * @brief Handle for a POST /management/extensions request.
 */
struct TALER_EXCHANGE_PostManagementExtensionsHandle
{

  /**
   * The base URL for this request.
   */
  char *base_url;

  /**
   * The full URL for this request, set during _start.
   */
  char *url;

  /**
   * Minor context that holds body and headers.
   */
  struct TALER_CURL_PostContext post_ctx;

  /**
   * Handle for the request.
   */
  struct GNUNET_CURL_Job *job;

  /**
   * Function to call with the result.
   */
  TALER_EXCHANGE_PostManagementExtensionsCallback cb;

  /**
   * Closure for @a cb.
   */
  TALER_EXCHANGE_POST_MANAGEMENT_EXTENSIONS_RESULT_CLOSURE *cb_cls;

  /**
   * Reference to the execution context.
   */
  struct GNUNET_CURL_Context *ctx;

  /**
   * Extension configuration data (we hold a reference).
   */
  json_t *extensions;

  /**
   * Signature over the extension configuration.
   */
  struct TALER_MasterSignatureP extensions_sig;

};


/**
 * Function called when we're done processing the
 * HTTP POST /management/extensions request.
 *
 * @param cls the `struct TALER_EXCHANGE_PostManagementExtensionsHandle`
 * @param response_code HTTP response code, 0 on error
 * @param response response body, NULL if not in JSON
 */
static void
handle_post_extensions_finished (void *cls,
                                 long response_code,
                                 const void *response)
{
  struct TALER_EXCHANGE_PostManagementExtensionsHandle *pmeh = cls;
  const json_t *json = response;
  struct TALER_EXCHANGE_PostManagementExtensionsResponse res = {
    .hr.http_status = (unsigned int) response_code,
    .hr.reply = json
  };

  pmeh->job = NULL;
  switch (response_code)
  {
  case MHD_HTTP_NO_CONTENT:
    break;
  case MHD_HTTP_FORBIDDEN:
    res.hr.ec = TALER_JSON_get_error_code (json);
    res.hr.hint = TALER_JSON_get_error_hint (json);
    break;
  case MHD_HTTP_NOT_FOUND:
    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                "Server did not find handler at `%s'. Did you configure the correct exchange base URL?\n",
                pmeh->url);
    if (NULL != json)
    {
      res.hr.ec = TALER_JSON_get_error_code (json);
      res.hr.hint = TALER_JSON_get_error_hint (json);
    }
    else
    {
      res.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
      res.hr.hint = TALER_ErrorCode_get_hint (res.hr.ec);
    }
    break;
  default:
    /* unexpected response code */
    GNUNET_break_op (0);
    res.hr.ec = TALER_JSON_get_error_code (json);
    res.hr.hint = TALER_JSON_get_error_hint (json);
    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                "Unexpected response code %u/%d for exchange management post extensions\n",
                (unsigned int) response_code,
                (int) res.hr.ec);
    break;
  }
  if (NULL != pmeh->cb)
  {
    pmeh->cb (pmeh->cb_cls,
              &res);
    pmeh->cb = NULL;
  }
  TALER_EXCHANGE_post_management_extensions_cancel (pmeh);
}


struct TALER_EXCHANGE_PostManagementExtensionsHandle *
TALER_EXCHANGE_post_management_extensions_create (
  struct GNUNET_CURL_Context *ctx,
  const char *url,
  const struct TALER_EXCHANGE_ManagementPostExtensionsData *ped)
{
  struct TALER_EXCHANGE_PostManagementExtensionsHandle *pmeh;

  pmeh = GNUNET_new (struct TALER_EXCHANGE_PostManagementExtensionsHandle);
  pmeh->ctx = ctx;
  pmeh->base_url = GNUNET_strdup (url);
  pmeh->extensions = json_incref ((json_t *) ped->extensions);
  pmeh->extensions_sig = ped->extensions_sig;
  return pmeh;
}


enum TALER_ErrorCode
TALER_EXCHANGE_post_management_extensions_start (
  struct TALER_EXCHANGE_PostManagementExtensionsHandle *pmeh,
  TALER_EXCHANGE_PostManagementExtensionsCallback cb,
  TALER_EXCHANGE_POST_MANAGEMENT_EXTENSIONS_RESULT_CLOSURE *cb_cls)
{
  CURL *eh = NULL;
  json_t *body = NULL;
  json_t *extensions;

  pmeh->cb = cb;
  pmeh->cb_cls = cb_cls;
  pmeh->url = TALER_url_join (pmeh->base_url,
                              "management/extensions",
                              NULL);
  if (NULL == pmeh->url)
  {
    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                "Could not construct request URL.\n");
    return TALER_EC_GENERIC_CONFIGURATION_INVALID;
  }
  /* Transfer our reference to the body via steal; set to NULL first */
  extensions = pmeh->extensions;
  pmeh->extensions = NULL;
  body = GNUNET_JSON_PACK (
    GNUNET_JSON_pack_object_steal ("extensions",
                                   extensions),
    GNUNET_JSON_pack_data_auto ("extensions_sig",
                                &pmeh->extensions_sig));
  eh = TALER_EXCHANGE_curl_easy_get_ (pmeh->url);
  if ( (NULL == eh) ||
       (GNUNET_OK !=
        TALER_curl_easy_post (&pmeh->post_ctx,
                              eh,
                              body)) )
  {
    GNUNET_break (0);
    if (NULL != eh)
      curl_easy_cleanup (eh);
    json_decref (body);
    GNUNET_free (pmeh->url);
    pmeh->url = NULL;
    return TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE;
  }
  json_decref (body);
  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
              "Requesting URL '%s'\n",
              pmeh->url);
  pmeh->job = GNUNET_CURL_job_add2 (pmeh->ctx,
                                    eh,
                                    pmeh->post_ctx.headers,
                                    &handle_post_extensions_finished,
                                    pmeh);
  if (NULL == pmeh->job)
  {
    TALER_curl_easy_post_finished (&pmeh->post_ctx);
    GNUNET_free (pmeh->url);
    pmeh->url = NULL;
    return TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE;
  }
  return TALER_EC_NONE;
}


void
TALER_EXCHANGE_post_management_extensions_cancel (
  struct TALER_EXCHANGE_PostManagementExtensionsHandle *pmeh)
{
  if (NULL != pmeh->job)
  {
    GNUNET_CURL_job_cancel (pmeh->job);
    pmeh->job = NULL;
  }
  TALER_curl_easy_post_finished (&pmeh->post_ctx);
  if (NULL != pmeh->extensions)
  {
    json_decref (pmeh->extensions);
    pmeh->extensions = NULL;
  }
  GNUNET_free (pmeh->url);
  GNUNET_free (pmeh->base_url);
  GNUNET_free (pmeh);
}


/* end of exchange_api_post-management-extensions.c */
